﻿/*
    ※Microsoft.Speech.dllへの参照設定が必要になります。通常は次のパスに存在します。
    「C:\Program Files\Microsoft Speech Platform SDK\Assembly\Microsoft.Speech.dll」
    
    『概要』
    特定のポーズを認識するサンプルです。またESM(EasySpeechManager)を利用した音声認識も行っています。
    より簡単なESMのサンプルは「SimpleESMSampleWithKinectプロジェクト」で確認することが出来ます。

    『機能』
    「save」が認識された場合は、その時のポーズが保存され、右の画面に表示されます。
    左の画面に映るリアルタイムのポーズが、登録したポーズと一致したと認識された場合、
    右側の画面の背景を黒くして通知します。「change」が認識された場合は、
    上半身の身を認識対象とするか、全身を認識対象とするか、認識対象を入れ替えます。

    EKLの内、主に次の機能が利用されています。
        - JointCollectionEx(TrackingScore,CheckWholeBodyPose,CheckUpperBodyPose, etc…)
        - KinectAudioSourceExtensions
        - SkeletonCanvas
    ESMの内、主に次の機能が利用されています。
        - EasyRecognitionManager
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

//追加
using Microsoft.Kinect;
using ExtendKinectLibrary;
using EasySpeechManager;
using Microsoft.Speech.Recognition;

namespace CheckPoseSample
{

    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {

        KinectSensor kinect;
        JointCollectionEx currentPose;
        JointCollectionEx targetPose;
        bool mode = true;

        KinectAudioSource kinectAudioSource;
        EasyRecognitionManager recognitionManager;
        private readonly string[] RecognitionWords = new string[] { "save", "change" };

        public MainWindow()
        {
            InitializeComponent();

            //Kinectが一台も繋がっていなかったら終了する。
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("Failed to find some Kinect sensor.");
                this.Close();
            }
            else
            {
                //Kinectの初期化と設定
                kinect = KinectSensor.KinectSensors[0];
                kinect.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(kinect_ColorFrameReady);
                kinect.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30);
                kinect.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(kinect_SkeletonFrameReady);
                kinect.SkeletonStream.Enable();

                //RecognitionManagerの初期化と設定
                try
                {
                    recognitionManager = new EasyRecognitionManager("en-us");

                    //認証させる語句の追加
                    recognitionManager.AddGrammar(RecognitionWords);

                    //KinectからAudioSourceを取得する
                    kinectAudioSource = kinect.AudioSource;
                    //Speechによる音声認識の為に最適化する
                    kinectAudioSource.OptimizeForSpeechRecognition();

                    //音声が認識された場合のイベントハンドラを追加
                    recognitionManager.Recognized += new EasyRecognitionManager.RecognizedEvent(recognitionManager_Recognized);

                    //Kinectの開始
                    kinect.Start();
                    //継続認証の開始
                    recognitionManager.MultipleStart(kinectAudioSource.Start());

                    //Windowイベントの設定
                    this.Closed += new EventHandler(MainWindow_Closed);
                    this.KeyDown += new KeyEventHandler(MainWindow_KeyDown);

                }
                //指定したカルチャ名が存在しない場合例外がスローされる
                catch (RecognizerInfoNotFoundException e)
                {
                    //メッセージを表示して終了する
                    string temp = "These are valid CultureName.\n";
                    foreach (string name in EasyRecognitionManager.GetCultureNameArray())
                        temp += "[" + name + "]\n";
                    MessageBox.Show(e.Message + "\n" + temp);

                    this.Close();
                }
            }
        }

        /// <summary>
        /// キーボードから入力された時の処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MainWindow_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Up:
                    kinect.TiltUp();
                    return;
                case Key.Down:
                    kinect.TiltDown();
                    return;
                case Key.Left:
                    kinect.SetTiltAngle(kinect.MaxElevationAngle);
                    return;
                case Key.Right:
                    kinect.SetTiltAngle(kinect.MinElevationAngle);
                    return;
                default:
                    return;
            }
        }

        /// <summary>
        /// ウィンドウが閉じられるときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MainWindow_Closed(object sender, EventArgs e)
        {
            //Kinectを停止してインスタンスを破棄する
            if(kinect != null)
                kinect.Close();
        }

        /// <summary>
        /// カメラのデータがKinectから流れて来たときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void kinect_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            ColorImageFrame temp = e.OpenColorImageFrame();

            if (temp != null)
            {
                //カメラ画像を表示する
                image1.Source = temp.GetBitmapSource();

                //ImageFrameを破棄する
                temp.Dispose();
            }
        }

        /// <summary>
        /// 骨格のデータがKinectから流れて来たときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void kinect_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {

            SkeletonFrame temp = e.OpenSkeletonFrame();

            if (temp != null)
            {
                //骨格データを貰う(ここでは一人しか映らないことを前提としている)
                Skeleton[] skeletonData = temp.GetSkeletonData();

                //JointManagerの更新
                foreach (Skeleton skeleton in skeletonData)
                {
                    //追跡できている骨格があったら
                    if (skeleton.TrackingState == SkeletonTrackingState.Tracked)
                    {
                        //現在のポーズを設定する(JointCollectionExへ暗黙の変換)
                        currentPose = skeleton.Joints;

                        //最新のJointとBoneを表示する
                        skeletonCanvas1.ShowJointAndBoneInColorImage(kinect, currentPose);
                        //最新のTrackingScoreを表示する
                        label1.Content = "TrackingScore:" + currentPose.TrackingScore;

                        //ポーズが設定されていなければ処理を終了する
                        if (targetPose == null)
                            continue;

                        //ポーズが一致するかチェックする
                        //一致してたら背景色を変更する
                        if (mode)
                        {
                            //全身を対象とする
                            if (currentPose.CheckWholeBodyPose(targetPose, 0.7F))
                                skeletonCanvas2.Background = new SolidColorBrush(Colors.Black);
                            else
                                skeletonCanvas2.Background = new SolidColorBrush(Colors.White);
                        }
                        else
                        {
                            //上半身の身を対象とする
                            if (currentPose.CheckUpperBodyPose(targetPose, 0.7F))
                                skeletonCanvas2.Background = new SolidColorBrush(Colors.Black);
                            else
                                skeletonCanvas2.Background = new SolidColorBrush(Colors.White);
                        }
                    }
                }

                //フレームを破棄
                temp.Dispose();
            }
        }

        /// <summary>
        /// 音声認識された場合のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void recognitionManager_Recognized(object sender, SpeechRecognizedEventArgs e)
        {
            switch(e.Result.Text){
                //「save」を認証したら現在の状態を認識するポーズに設定する
                case "save":
                {
                    targetPose = currentPose;
                    skeletonCanvas2.ShowJointAndBoneInColorImage(kinect, targetPose);
                    break;
                }
                //「change」を認証したら認識のモードを切り替える
                case "change":
                {
                    mode = !mode;
                    if (mode)
                        label2.Content = "Mode:WholeBody";
                    else
                        label2.Content = "Mode:UpperBody";
                    break;
                }            
            };
        }
    }

}
