﻿/*
    ※Microsoft.Speech.dllへの参照設定が必要になります。通常は次のパスに存在します。
    「C:\Program Files\Microsoft Speech Platform SDK\Assembly\Microsoft.Speech.dll」
    
    『概要』
    ユーザの動きを録画して再生するサンプルです。Skeletonデータの書き出し、読み込みも行っています。
    またESM(EasySpeechManager)を利用した音声認識も行っています。より簡単なESMのサンプルは
    「SimpleESMSampleWithKinectプロジェクト」で確認することが出来ます。

    『機能』
    「record」という単語が認識されると、10秒の間、ユーザの動作を録画してCSVファイルとして出力します。
    「play」という単語が認識されると、CSVファイルからユーザの動作を読み取って再生します。

    EKLの内、主に次の機能が利用されています。
        - KinectAudioSourceExtensions
        - SkeletonCanvas
        - JointManager
        - SkeletonReader
        - SkeletonWriter
    ESMの内、主に次の機能が利用されています。
        - EasyRecognitionManager
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

//追加
using ExtendKinectLibrary;
using Microsoft.Kinect;
using Microsoft.Speech.Recognition;
using EasySpeechManager;
using System.Windows.Threading;

namespace MotionCaptureSample
{

    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {

        KinectSensor kinect;
        JointCollectionManager jcManager;

        KinectAudioSource kinectAudioSource;
        EasyRecognitionManager recognitionManager;
        private readonly string[] RecognitionWords = new string[] { "record", "play" };

        //録画用タイマ
        DispatcherTimer recordTimer;
        //保存中かどうか
        bool nowRecording = false;
        //最大録画時間
        const int MaxRecordeTime = 10;
        //現在の時間
        int currentTime = 0;

        //再生用タイマ
        DispatcherTimer playTimer;
        //現在再生中かどうか
        bool nowPlaying = false;
        //現在の再生フレーム
        int currentFrame = 0;

        public MainWindow()
        {
            InitializeComponent();

            //Kinectが一台も繋がっていなかったら終了する。
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("Failed to find some Kinect sensor.");
                this.Close();
            }
            else
            {
                //Kinectの初期化と設定
                kinect = KinectSensor.KinectSensors[0];
                kinect.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(kinect_SkeletonFrameReady);
                kinect.SkeletonStream.Enable();

                //JointManagerの初期化と設定
                jcManager = new JointCollectionManager(-1);

                //RecognitionManagerの初期化と設定
                try
                {
                    recognitionManager = new EasyRecognitionManager("en-us");

                    //認証させる語句の追加
                    recognitionManager.AddGrammar(RecognitionWords);

                    //KinectからAudioSourceを取得する
                    kinectAudioSource = kinect.AudioSource;
                    //Speechによる音声認識の為に最適化する
                    kinectAudioSource.OptimizeForSpeechRecognition();

                    //音声が認識された場合のイベントハンドラを追加
                    recognitionManager.Recognized += new EasyRecognitionManager.RecognizedEvent(recognitionManager_Recognized);

                    //Kinectの開始
                    kinect.Start();
                    //継続認証の開始
                    recognitionManager.MultipleStart(kinectAudioSource.Start());

                    //Windowイベントの設定
                    this.Closed += new EventHandler(MainWindow_Closed);
                    this.KeyDown += new KeyEventHandler(MainWindow_KeyDown);

                    //録画タイマを初期化
                    recordTimer = new DispatcherTimer();
                    //録画時間を設定(1sec毎)
                    recordTimer.Interval = TimeSpan.FromSeconds(1);
                    //タイマイベントを設定
                    recordTimer.Tick += new EventHandler(recordTimer_Tick);

                    //再生タイマを初期化
                    playTimer = new DispatcherTimer();
                    //タイマイベントを設定
                    playTimer.Tick += new EventHandler(playTimer_Tick);
                }

                //指定したカルチャ名が存在しない場合例外がスローされる
                catch (RecognizerInfoNotFoundException e)
                {
                    //メッセージを表示して終了する
                    string temp = "These are valid CultureName.\n";
                    foreach(string name in EasyRecognitionManager.GetCultureNameArray())
                        temp += "[" + name + "]\n";
                    MessageBox.Show(e.Message + "\n" + temp);
                    
                    this.Close();
                }

               
            }
        }

        /// <summary>
        /// 再生中に呼び出されるイベントハンドラ。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void playTimer_Tick(object sender, EventArgs e)
        {
            //再生が終了したら
            if (currentFrame == jcManager.Capacity - 1)
            {
                //再生を停止する
                playTimer.Stop();
                nowPlaying = false;

                //表示を戻す
                label1.Content = MaxRecordeTime.ToString();
                
                //現在のフレームを元に戻す
                currentFrame = 0;

                return;
            }
            
            //JointManagerから対象のフレームのJointCollectionEを貰って表示する
            skeletonCanvas.ShowJointAndBoneInColorImage(
                kinect,jcManager.GetJointCollectionE(currentFrame));
            
            //現在のフレームを更新する
            currentFrame += 1;
            
            //次に呼び出す時間を変更する
            playTimer.Interval = TimeSpan.FromMilliseconds(
                jcManager.GetJointCollectionE(currentFrame).TimeStamp
                - jcManager.GetJointCollectionE(currentFrame - 1).TimeStamp);
        }

        /// <summary>
        /// 録画中に1秒おきに発生するイベントハンドラ。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void recordTimer_Tick(object sender, EventArgs e)
        {
            //録画時間を1足す
            currentTime += 1;

            //録画時間を更新する
            label1.Content = (MaxRecordeTime - currentTime).ToString();

            //録画終了時間になったら
            if (currentTime == MaxRecordeTime)
            {
                //録画を中止する
                nowRecording = false;
                recordTimer.Stop();
                
                //録画時間を初期化する
                currentTime = 0;
                label1.Content = MaxRecordeTime.ToString();
                
                //各JointCollectionの時間を録画開始時間に調整する。
                jcManager.SetStartTimeFromZero();

                //ファイルに出力する
                SkeletonWriter.WriteJointsInCSV(
                    "temp.csv",
                    Encoding.UTF8,
                    false,
                    this.jcManager.GetJointCollectionEArray(),
                    SkeletonDataType.IncludeTrackingState);
            }
        }

        /// <summary>
        /// キーボードから入力された時の処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MainWindow_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Up:
                    kinect.TiltUp();
                    return;
                case Key.Down:
                    kinect.TiltDown();
                    return;
                case Key.Left:
                    kinect.SetTiltAngle(kinect.MaxElevationAngle);
                    return;
                case Key.Right:
                    kinect.SetTiltAngle(kinect.MinElevationAngle);
                    return;
                default:
                    return;
            }
        }

        /// <summary>
        /// ウィンドウが閉じられるときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MainWindow_Closed(object sender, EventArgs e)
        {
            //Kinectを停止してインスタンスを破棄する
            if (kinect != null)
                kinect.Close();
        }

        /// <summary>
        /// 骨格のデータがKinectから流れて来たときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void kinect_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {

            SkeletonFrame temp = e.OpenSkeletonFrame();

            if (temp != null)
            {
                //骨格データを貰う(ここでは一人しか映らないことを前提としている)
                Skeleton[] skeletonData = temp.GetSkeletonData();

                //JointManagerの更新
                foreach (Skeleton skeleton in skeletonData)
                {
                    //追跡できている骨格があったら
                    if (skeleton.TrackingState == SkeletonTrackingState.Tracked)
                    {
                        //録画中ならJointManagerを更新する
                        if(nowRecording)
                            jcManager.Update(temp.Timestamp,skeleton.Joints);

                        //再生中でないなら最新のJointとBoneを表示する
                        if (!nowPlaying)
                            skeletonCanvas.ShowJointAndBoneInColorImage(kinect,skeleton.Joints);
                    }
                }

                //フレームを破棄
                temp.Dispose();
            }
        }

        /// <summary>
        /// 音声認識された場合のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void recognitionManager_Recognized(object sender, SpeechRecognizedEventArgs e)
        {
            switch (e.Result.Text)
            {
                //「record」を認識したら録画を開始する
                case "record":
                    {
                        //再生中か録画中なら処理をしない
                        if (nowRecording || nowPlaying)
                            return;

                        //録画を開始する
                        jcManager = new JointCollectionManager(-1);
                        label1.Content = MaxRecordeTime;
                        nowRecording = true;
                        recordTimer.Start();
                        break;
                    }
                //「play」を認識ファイルを読み取って再生を開始する
                case "play":
                    {
                        //再生中か録画中なら処理をしない
                        if (nowRecording || nowPlaying)
                            return;

                        //CSVの読み取り
                        jcManager = new JointCollectionManager(
                            SkeletonReader.ReadJointsArrayFromCSV(
                            "temp.csv",
                            Encoding.UTF8,
                            SkeletonDataType.IncludeTrackingState)
                            );

                        //再生する時間を決定する
                        playTimer.Interval = TimeSpan.FromMilliseconds
                            (jcManager.GetJointCollectionE(1).TimeStamp
                            - jcManager.GetJointCollectionE(0).TimeStamp);
                        //再生タイマを開始する
                        nowPlaying = true;
                        playTimer.Start();
                        //表示内容を変更する
                        label1.Content = "NowPlaying.";
                        break;
                    }
            };
        }

    }

}


