﻿/*
    『概要』
    身長と左腕の角度を表示するサンプルです。映し出された"1人"のユーザの身長を表示します。
    正確性を優先しない手法のため、全身がきれいに映らなければ、平気で10cm近くずれます。
    WPFアプリケーションで素早くユーザの骨格を可視化する場合「SkeletonCanvas」が利用できます。
    SkeletonCanvasは「CheckPoseSampleプロジェクト」などから確認することが出来ます。

    EKLの内、主に次の機能が利用されています。
        - JointCollectionEx(Bone系プロパティ,Angle系プロパティ)
        - Bone
        - BoneCollection
        - KinectSensorExtensions
        - SkeletonPointExtensions
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

//追加
using Microsoft.Kinect;
using ExtendKinectLibrary;

namespace ShowBodyInfoSample
{

    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {

        KinectSensor kinect;
        JointCollectionEx jointCollectionEx;

        SolidColorBrush JointPointBrush = new SolidColorBrush(Colors.Blue);
        SolidColorBrush BoneLineBrush = new SolidColorBrush(Colors.LightSeaGreen);
        SolidColorBrush TextBrush = new SolidColorBrush(Colors.Red);
        Ellipse JointEllipse
        {
            get
            {
                Ellipse temp = new Ellipse();
                temp.Fill = JointPointBrush;
                temp.Height = 20;
                temp.Width = 20;
                return temp;
            }
        }
        Line BoneLine
        {
            get
            {
                Line temp = new Line();
                temp.StrokeThickness = 10;
                temp.Stroke = BoneLineBrush;
                return temp;
            }
        }
        Label DataLabel
        {
            get
            {
                Label temp = new Label();
                temp.FontSize = 28;
                temp.Foreground = TextBrush;
                temp.FontWeight = FontWeights.ExtraBold;
                Color back = Colors.White;
                back.A = 230;
                temp.Background = new SolidColorBrush(back);
                return temp;
            }
        }

        public MainWindow()
        {
            InitializeComponent();

            //Kinectが一台も繋がっていなかったら終了する。
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("Failed to find some Kinect sensor.");
                this.Close();
            }
            else
            {
                //Kinectの初期化と設定
                kinect = KinectSensor.KinectSensors[0];
                kinect.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(kinect_ColorFrameReady);
                kinect.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30);
                kinect.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(kinect_SkeletonFrameReady);
                kinect.SkeletonStream.Enable();
                kinect.Start();

                //Windowイベントの設定
                this.Closed += new EventHandler(MainWindow_Closed);
                this.KeyDown += new KeyEventHandler(MainWindow_KeyDown);
            }
        }

        /// <summary>
        /// キーボードから入力された時の処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MainWindow_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Up:
                    kinect.TiltUp();
                    return;
                case Key.Down:
                    kinect.TiltDown();
                    return;
                case Key.Left:
                    kinect.SetTiltAngle(kinect.MaxElevationAngle);
                    return;
                case Key.Right:
                    kinect.SetTiltAngle(kinect.MinElevationAngle);
                    return;
                default:
                    return;
            }
        }

        /// <summary>
        /// ウィンドウが閉じられるときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MainWindow_Closed(object sender, EventArgs e)
        {
            //Kinectを停止してインスタンスを破棄する
            if(kinect != null)
                kinect.Close();
        }

        /// <summary>
        /// カメラのデータがKinectから流れて来たときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void kinect_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            ColorImageFrame temp = e.OpenColorImageFrame();

            if (temp != null)
            {
                //カメラ画像を表示する
                image1.Source = temp.GetBitmapSource();

                //ImageFrameを破棄する
                temp.Dispose();
            }
        }

        /// <summary>
        /// 骨格のデータがKinectから流れて来たときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void kinect_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {

            SkeletonFrame temp = e.OpenSkeletonFrame();

            if (temp != null)
            {
                //骨格データを貰う(ここでは一人しか映らないことを前提としている)
                Skeleton[] skeletonData = temp.GetSkeletonData();

                //BoneManagerの更新
                foreach (Skeleton skeleton in skeletonData)
                {
                    //追跡できている骨格があったら
                    if (skeleton.TrackingState == SkeletonTrackingState.Tracked)
                    {
                        //JointCollectionをJointCollectionExへ(暗黙の型変換を実行する)
                        jointCollectionEx = skeleton.Joints;

                        //JointとBoneを表示する
                        ShowJointAndBone(jointCollectionEx);
                        //疑似的な身長を表示する
                        ShowPlayerHeight(jointCollectionEx);
                        //左腕の角度を表示する
                        ShowLeftArmAngle(jointCollectionEx);
                    }
                }

                //フレームを破棄
                temp.Dispose();
            }
        }

        /// <summary>
        /// JointとBoneの位置をキャンバス上に表示する。
        /// </summary>
        void ShowJointAndBone(JointCollectionEx joints)
        {
            //キャンバスの初期化
            canvas.Children.Clear();
            //Boneの表示
            ShowBoneLine(joints);
            //Jointの表示(Jointが上に来る方が見栄えが良い)
            ShowJointPoint(joints);
        }

        /// <summary>
        /// Jointの位置を表示する。
        /// </summary>
        void ShowJointPoint(JointCollectionEx joints)
        {
            foreach (Joint joint in joints)
            {
                //カラー画像での座標を取得する
                ColorImagePoint position = joint.Position.InColorImage(kinect);
                //Jointを示すEllipse
                Ellipse temp = JointEllipse;
                //座標を設定して(点の半径分ずらす)
                Canvas.SetLeft(temp, position.X - temp.Width / 2);
                Canvas.SetTop(temp, position.Y - temp.Height / 2);
                //キャンバスに追加
                canvas.Children.Add(temp);
            }
        }

        /// <summary>
        /// Boneの線を表示する。
        /// </summary>
        void ShowBoneLine(JointCollectionEx joints)
        {
            //最新のフレームに含まれるBone全てに対して処理を行う。
            //※JointCollectionから得られる全てのBoneに対し、
            //再起的な処理を行いたい場合はBoneCollectionを利用します。
            //それ以外の場合は、該当のBoneを取得する方が効率的です。
            foreach (Bone bone in joints.BoneCollection)
            {
                //骨の両端の座標を取得する
                ColorImagePoint root = bone.RootJoint.Position.InColorImage(kinect);
                ColorImagePoint end = bone.EndJoint.Position.InColorImage(kinect);
                //骨を示すLine
                Line temp = BoneLine;
                //座標を設定して
                temp.X1 = root.X;
                temp.Y1 = root.Y;
                temp.X2 = end.X;
                temp.Y2 = end.Y;
                //キャンバスに追加
                canvas.Children.Add(temp);
            }
        }

        /// <summary>
        /// 認識したプレイヤ(このアプリケーションは簡単のため1人を前提としている)の身長を表示する。
        /// </summary>
        void ShowPlayerHeight(JointCollectionEx joints)
        {
            //ラベルを作って
            Label temp = DataLabel;
            //ラベルの表示内容を設定する
            temp.Content = "▼[" + (int)CalcHeight(joints) + "]";

            //頭の位置を拾ってきて
            Joint head = joints[JointType.Head];
            //カラー画像での位置に変換する
            ColorImagePoint headPosition = head.Position.InColorImage(kinect);

            //ラベルを頭の横に来るように設定して
            Canvas.SetTop(temp, headPosition.Y);
            Canvas.SetLeft(temp, headPosition.X + 20);

            //キャンバスに追加
            canvas.Children.Add(temp);
        }

        /// <summary>
        /// 疑似的な身長を計算して返す。かなり曖昧。
        /// </summary>
        /// <param name="boneCollection"></param>
        /// <returns></returns>
        float CalcHeight(JointCollectionEx joints)
        {
            float height = 0;
            height += joints.Bone_ShoulderC_Head.Length;
            height += joints.Bone_Spine_ShoulderC.Length;
            height += joints.Bone_HipC_Spine.Length;
            height += joints.Bone_HipC_HipL.Length;
            height += joints.Bone_HipL_KneeL.Length;
            height += joints.Bone_KneeL_AnkleL.Length;
            return height * 100 + 13;//mをcmに変換する。頭の半分のサイズだけ足す(日本人平均)。
        }

        /// <summary>
        /// 左腕の角度を表示する。
        /// </summary>
        void ShowLeftArmAngle(JointCollectionEx joints)
        {
            //ラベルを作って
            Label temp = DataLabel;
            //ラベルの表示内容を設定する
            //「左肩から左ひじ」への骨と「左ひじから左手首」への骨の角度を取得する
            temp.Content = "▼[" + (int)(joints.Angle_LeftElbow * 180 / Math.PI) + "]";

            //左ひじの位置を拾ってきて、カラー画像での位置に変換する
            ColorImagePoint elbowLPosition = joints[JointType.ElbowLeft]
                .Position.InColorImage(kinect);

            //ラベルを左ひじの横に来るように設定して
            Canvas.SetTop(temp, elbowLPosition.Y);
            Canvas.SetLeft(temp, elbowLPosition.X + 20);

            //キャンバスに追加
            canvas.Children.Add(temp);
        }

    }

}
