﻿/*
    ※Microsoft.Speech.dllへの参照設定が必要になります。通常は次のパスに存在します。
    「C:\Program Files\Microsoft Speech Platform SDK\Assembly\Microsoft.Speech.dll」
    
    『概要』
    EasySpeechManager(ESM)を利用して、音声認識によってKinectを操作するサンプルです。

    『機能』
    「Up」が認識された場合は、Kinectをチルトアップします。
    「Down」が認識された場合は、Kinectをチルトダウンします。

    EKLの内、主に次の機能が利用されています。
        - KinectSensorExtensions(TiltUp,TiltDown)
    ESMの内、主に次の機能が利用されています。
        - EasyRecognitionManager
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

//追加
using Microsoft.Kinect;
using ExtendKinectLibrary;
using EasySpeechManager;
using Microsoft.Speech.Recognition;

namespace SimpleESMSampleWithKinect
{

    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {

        KinectSensor kinect;
        KinectAudioSource kinectAudioSource;
        EasyRecognitionManager recognitionManager;

        private readonly string[] tiltWords = new string[] { "up", "down" };

        public MainWindow()
        {
            InitializeComponent();

            //Kinectが一台も繋がっていなかったら終了する。
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("Failed to find some Kinect sensor.");
                this.Close();
            }
            else
            {
                //Kinectの初期化と設定
                kinect = KinectSensor.KinectSensors[0];
                kinect.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(kinect_ColorFrameReady);
                kinect.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30);

                //RecognitionManagerの初期化と設定
                //デバッグ用に、インストールされたカルチャー名を一覧する
                foreach (string name in EasyRecognitionManager.GetCultureNameArray())
                    Console.WriteLine(name);
                try
                {
                    recognitionManager = new EasyRecognitionManager("en-us");

                    //認証させる語句の追加
                    recognitionManager.AddGrammar(tiltWords);

                    //KinectからAudioSourceを取得する
                    kinectAudioSource = kinect.AudioSource;
                    //Speechによる音声認識の為に最適化する
                    kinectAudioSource.OptimizeForSpeechRecognition();

                    //音声が認識された場合のイベントハンドラを追加
                    recognitionManager.Recognized += new EasyRecognitionManager.RecognizedEvent(recognitionManager_Recognized);

                    //Kinectの開始
                    kinect.Start();
                    //継続認証の開始
                    recognitionManager.MultipleStart(kinectAudioSource.Start());
                    //ラベルの初期化
                    this.label1.Content = "Angle:" + kinect.ElevationAngle;

                    //Windowイベントの設定
                    this.Closed += new EventHandler(MainWindow_Closed);

                }
                //指定したカルチャ名が存在しない場合例外がスローされる
                catch (RecognizerInfoNotFoundException e)
                {
                    //メッセージを表示して終了する
                    string temp = "These are valid CultureName.\n";
                    foreach (string name in EasyRecognitionManager.GetCultureNameArray())
                        temp += "[" + name + "]\n";
                    MessageBox.Show(e.Message + "\n" + temp);

                    this.Close();
                }
            }
        }

        /// <summary>
        /// 音声認識された場合のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void recognitionManager_Recognized(object sender, SpeechRecognizedEventArgs e)
        {
            switch (e.Result.Text)
            {
                case "up":
                        kinect.TiltUp();
                        break;
                case "down":
                        kinect.TiltDown();
                        break;
            }
            this.label1.Content = "Angle:" + kinect.ElevationAngle;
        }

        /// <summary>
        /// ウィンドウが閉じられるときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MainWindow_Closed(object sender, EventArgs e)
        {
            //Kinectを停止してインスタンスを破棄する
            if(kinect != null)
                kinect.Close();
        }

        /// <summary>
        /// カメラのデータがKinectから流れて来たときの処理。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void kinect_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            ColorImageFrame temp = e.OpenColorImageFrame();

            if (temp != null)
            {
                //カメラ画像を表示する
                image1.Source = temp.GetBitmapSource();

                //ImageFrameを破棄する
                temp.Dispose();
            }
        }

    }

}
